<?php

// -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
/**
 *   Paloose: Cocoon without Java
 *
 *   Paloose is a Cocoon variant written entirely in PHP without Java.
 *   It resulted from scratching a long standing personal itch: that there are very few
 *   ISPs who will support Java/Tomcat for web sites, other than as a very expensive
 *   "professional" addition. Almost all will support PHP5 (sorry, Paloose does not use PHP4)
 *   and so I decided to write my version of a simple, cut-down Cocoon in PHP5. I have been using
 *   Cocoon for some time now and have always found it the best way to build XML/XSL
 *   based sites - is there any other way? :-)
 *   
 *   The code that builds Paloose is a little rough and ready in places. This is my
 *   first PHP program (other than the odd embedded line within HTML) and it shows. I am
 *   more used to program in strongly typed languages having had a lifetime using
 *   Algol, Coral, Algol68, Pascal, Java at al. I have programmed in Perl for many
 *   years and so PHP was not a wholly new experience. I am sure that I have used 10 lines
 *   in places where PHP5 would let me use one; never mind - it suffices, the future will
 *   refine the code.
 *   
 *   "<i>The villiany you teach me I will execute, and it will go hard
 *    but I will better the instruction.</i>" William Shakespeare, The Merchant of Venice Act III Scene 1
 *
 * LICENSE:
 * 
 * Paloose is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program.  If not, see <http://www.gnu.org/licenses/> 
 *
 * @package paloose
 * @author Hugh Field-Richards <hsfr@hsfr.org.uk>
 * @version 1.3.4
 * @license http://www.opensource.org/licenses/lgpl-license.php LGPL
 * @copyright 2006 - 2009 Hugh Field-Richards
 */

/*
-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

   Version   Date                   Who    Changes
   ==========================================================================

   0.0.0    27th March 2006         HSFR   Created
   0.0.1    10th April 2006         HSFR   Major change to all to use log4php
   0.0.2    19th April 2006         HSFR   Change directory structure and start adding error/exceptions properly
   0.1.0    24th May 2006           HSFR   First issue (Hop Vine site and Paloose site both use it)
   0.2.0    13th July 2006          HSFR   Views mechanism sorted to work on more components
   0.3.0    20th July 2006          HSFR   i18n translation compeletely rewritten (handles dates (not time yet) )
   0.4.0    1st August 2006         HSFR   PageHit and Gallery Transformer added
   0.5.0    12th August 2006        HSFR   Preliminary TidyGenerator added
   0.5.1    13th August 2006        HSFR   Changed base class for WildcardURIMatcherException
   0.5.2    21st August 2006        HSFR   Added XInclude in FileGenerator
   0.6.0    12th September 2006     HSFR   Added browser selector
   0.6.1    4th October 2006        HSFR   Date/time localisation changes
   0.6.2    16th October 2006       HSFR   Minor cosmetic changes
   0.6.3    19th October 2006       HSFR   Minor change to the way date is processed in I18nTransformer.
   0.7.0    24th October 2006       HSFR   Added RegexpURIMatcher.
                                           Cleaned up use of matchers.
                                           Changed WildcardURIException to be a more general matcher exception, MatcherException.
                                           Cleaned up comments that were misleading (still some more to sort out unfortunately).
   0.8.0    26th October 2006       HSFR   Updated HTMLSerializer
                                           Added XHTMLSerializer, TextSerializer, XMLSerialzer.
   0.9.0    2nd November 2006       HSFR   Added Resources/Call mechanism.
                                           Cleaned up several areas of bad code and documentation.
                                           Cured several bugs in the error reporting.
                                           Made sure all pipe building components used same creation process in PipeElement class.
   0.9.1    15th November 2006      HSFR   Temporarily removed PipeElementInterface.
   0.9.2    16th November 2006      HSFR   Restored PipeElementInterface with suitable corrections to dependant classes.
   0.9.2b1  17th November 2006      HSFR   Strong PHP checking allowed for (and all subsequent betas at this level)
   ...
   0.9.2b9  20th November 2006      HSFR 
   0.10.0   21st November 2006      HSFR   Major work making the code more robust after turning on local server full PHP error reporting
   0.10.1   22nd November 2006      HSFR   Corrected uninitialised variable $badip in PageHitTransformer
   0.11.0   5th December 2006       HSFR   Added POST information into RequestParameterModule.
                                           Removed global variable for error messages (now passed through exceptions).
                                           Added SendMail action (Action, Actions, SendMailAction classes added).
                                           Cured bug in ErrorPage that occasionally gave errors in display.
                                           Many minor cosmetic changes.
                                           Major code documentation changes (still more to do).
   0.11.1   5th December 2006       HSFR   global $gConfiguration missing in TidyGenerator.
   0.12.0   10th December 2006      HSFR   Converted to use output buffer
                                           Added redirect-to pipeline element
                                           More code documentation changes
                                           Added internal-only pipelines
   0.13.0   December 2006           HSFR   Major clean up of documentation and code
   0.14.0   11th January 2007       HSFR   Adding authorisation/session/continuations mechanism (not complete)
   0.14.1   16th January 2007       HSFR   Cured minor bug in ResourceReader (unset variable).
                                           Cured bug in Mount where sitemap variables were not expanded.
                                           Cured bug in I18nTransformer (wrong check, unset variable in time evaluate).
   0.15.0   12th February 2007      HSFR   Changed the way subsitemap declarations work. You can now declare
                                           components within appropriate subsitemap, not always put them in the master sitemap
                                           Some minor documentation changes.
   1.0.0b1  26th March 2007         HSFR   Added PXTemplateGen generator for expanding embedded sitemap variables.
                                           Added SourceWriteTransformer for diverting XML to external file (cf Cocoon).
                                           Added PasswordTransformer for encrypting passwords in Authorisation framework.
                                           Added Authorisation actions: AuthAction, AuthenticationActions, LoginAction and LogoutAction.
                                           Added Paloose Form framework.
                                           Added Paloose Continuations/Flow framework.
                                           Cured problem where sitemap pseudo-protocols parameters were not being expanded.
                                           More minor code documentation updates.
   1.0.0b2  27th March 2007         HSFR   Added LogTransformer for pipeline logging.
   1.0.0b3  30th March 2007         HSFR   Created new class hierarchy for the pipeline elements. Only of note if
                                           you have created your own pipeline components. The inheritance is now
                                           from an intermediate class not the PipeElement class. For example
                                           TransformerPipeElement for all transformers. This affects generators,
                                           actions, transformers, selectors, readers, and serializers.
                                           Cured bug in Readers that would cause exception on duplicate Reader name in
                                           subsitemaps, rather than redefine.
   1.0.0b4  4th April 2007          HSFR   Cured bug in GalleryTransformer.
   1.0.0b5  23rd April 2007         HSFR   Made error reporting a little cleaner for XML/XSLT problems (not well formed etc).
                                           Minor cosmetics.
   1.0.0b6  24th April 2007         HSFR   Corrected some untidiness in the base classes for components (Transformers, Generators etc).
   1.0.0b7  2nd May 2007            HSFR   Added DirectoryGenerator.
            29th July 2007          HSFR   Better error reporting in WildcardURIMatcher
   1.0.1    11th August 2007        HSFR   Checking for null description in GalleryTransformer
   1.0.3    15th November 2007      HSFR   Added RequestParameterSelector
                                           Cured bug in _BrowserSelector (wrong name for getBrowserName method)
   1.0.4    18th December 2007      HSFR   Added support for ImageMagick if no PHP5 support for jpeg, png, gif.
   1.0.5    25th December 2007      HSFR   Added better error reporting in GalleryTransformer when image file does not exist.
   1.0.6    13th May 2008           HSFR   Added better browser checking
                                           Added iPhone support
   1.0.7    20th May 2008           HSFR   Cured toString() in PipeElement producing malformed XML
                                           Cured problem in SelectorPipeElement with nested select constructions.  
   1.1.0    3rd June 2008           HSFR   Cured discrepancy between Cocoon and Paloose in call statements. 
   1.1.1    14th June 2008          HSFR   Cured bug in Aggregator stopping context:// pseudo-protocols from working.
                                           Cured namespace problem in Galleries (must have namespace in gallery.xml)
   1.1.1b1  25th August 2008        HSFR   Minor cosmetics and included (undocumented) Gedcom Generator support.
                                           Cured bug in Aggregator stopping context:// pseudo-protocols from working.
                                           Cured namespace problem in Galleries (must have namespace in gallery.xml)
   1.1.1b2  5th October 2008        HSFR   Cured bug where ImageMagick files had fixed bin path
   1.1.1b3  6th October 2008        HSFR   Tidied up I18nTransformer where I18n namespace was being used on attributes
                                           Improved I18nTransformer default time.date patterns
   1.1.2    15th October 2008       HSFR   Release GedCom support
   1.1.2b1  26th November 2008      HSFR   Removed Log4PHP from Paloose distribution
   1.2.0    20th December 2008      HSFR   Added FilterTransformer
                                           Added SQLTransformer
                                           Added ParameterModule
                                           Added GlobalModule
                                           Numerous minor updates
                                           Add documentation and correct many mistakes within documentation
   1.2.0b1  21st December 2008      HSFR   Cleaned up documentation for new Classes and removed spurious methods
                                           Changed error numbering
   1.3.0    4th January 2009        HSFR   Added caching system (TRAXTransformer, Aggregator, GedComGenerator and FileGenerator)
                                           Changed Exception system constants to PalooseException only
                                           Cured append bug in LogTransformer
                                           Improved error reporting in Match.php
                                           Cured RequestParameter bug that prevent store of query parameters
                                           Removed parameter detection into PipeElement base class
                                           Removed label detection into PipeElement base class
                                           Additional PHPDocumentation comments
                                           Added file locking in page hit code - should have been there on day 1 :-(
                                           General code cleaning
   1.3.0b1  14th January 2009       HSFR   cure bug in GedComGenerator that prevented cache file being accessed correctly
   1.3.0b2  22nd January 2009       HSFR   cure bug in PageHitTransformer that resulted from locking
   1.3.0b3  5th February 2009       HSFR   cure bug in FileGenerator, PXTemplateGenerator and GedComGenertaor with undefined variable
                                           cured bug in TraxTransformer with undefined variables
   1.3.0b4  11th February 2009      HSFR   Sorted out several uninitialised variables
                                           Develop source with Eclipse instead of oXygen from now on
   1.3.1    18th February 2009      HSFR   Added XIncludeTransformer
                                           Removed xinclude stuff from the generators (use a tranformer now)
   1.3.1b1  24th February 2009      HSFR   Cured problem of namespaces in XHTMLSerializer
   1.3.2    5th March 2009          HSFR   Added EntityTransformer (only very beta at this stage)
   1.3.3    24th May 2009           HSFR   Added removal of Paloose attributes from the HTML/XHTML serializers (__xxxxx)
                                           Corrected Reader error-number bug
                                           Added GALLERY_ERROR
                                           Added some more logging lines in Pipeline.php
                                           Created ErrorModule.php and extended HandleError.php to handle errors better
   1.3.4    5th June 2009           HSFR   Added entity translation in generators (File and PXTemplate)
                                           Added regular expression variables to global module
   1.3.4b1  15th July 2009          HSFR   Cleaned up XHTMLSerializer
   1.3.4b2  22nd July 2009          HSFR   Changed entity translation to ignore "lt", "gt", "amp" and "quote" which are the
                                           standard HTML supported entities.
*/

$_SERVER[ 'DOCUMENT_ROOT' ] = substr( $_SERVER[ 'SCRIPT_FILENAME'], 0, strrpos( $_SERVER[ 'SCRIPT_FILENAME'], "/" ) + 1 );

// error_reporting( E_ERROR );      // Only output errors at present - mature system should only report through Paloose exceptions :-)
ini_set( 'track_errors', '1' );

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
// Do not change anything in this file. All changes are done in the calling
// file (paloose.php) in your web site folder which sets PALOOSE_DIRECTORY.

/**  @ignore  */
define( 'PALOOSE_DIR', $_SERVER[ 'DOCUMENT_ROOT' ] . PALOOSE_DIRECTORY );

/**  @ignore  */
define( 'PHP_DIR', PALOOSE_DIR . '/lib' );

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
// Will need this to resolve all pseudo-protocols below
require_once( PHP_DIR . "/environment/StringResolver.php" );

// Prime the $gConfiguration array with user requirements
require_once( PHP_DIR . "/config.inc.php" );

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
// The environment class is a useful means of providing context information and
// common useful methods for the environment processing.
require_once( PHP_DIR . "/environment/Environment.php" );
require_once( PHP_DIR . "/environment/Utilities.php" );
Environment::$gQueryString = $_SERVER[ 'QUERY_STRING' ];

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
// Prime the logging system (using log4php). Must be done outside any class
// to make it available to all defined classes. The default directory is to be
// the one included with the distribution. If you want to put log4php in
// another directory (you may already use it) define it in your "paloose.php"
// file in your site's root directory (see documentation).

/**  @ignore  */
if ( !defined( 'LOG4PHP_DIR' ) ) define( 'LOG4PHP_DIR', PHP_DIR . '/log4php' );
/**  @ignore  */
define( 'LOG4PHP_CONFIGURATION', StringResolver::expandPseudoProtocols( $gConfiguration[ 'loggerConfig' ] ) );
include_once( LOG4PHP_DIR . '/LoggerManager.php' );

$Logger =& LoggerManager::getLogger( 'main' );
$Logger->info( '==================================================');
$Logger->info( 'Raw Request URI: ' . $_SERVER[ 'REQUEST_URI' ] );
$Logger->info( 'Raw Query string: ' . $_SERVER[ 'QUERY_STRING' ] );
$Logger->info( 'Remote address: ' . $_SERVER[ 'REMOTE_ADDR' ] );
$Logger->debug( 'DOCUMENT_ROOT: ' . $_SERVER[ 'DOCUMENT_ROOT' ] );
$Logger->debug( 'PALOOSE_DIR: ' . PALOOSE_DIR );
$Logger->debug( 'PHP_DIR: ' . PHP_DIR );
$Logger->debug( 'LOG4PHP_DIR: ' . LOG4PHP_DIR );
$Logger->debug( 'LOG4PHP_CONFIGURATION: ' . LOG4PHP_CONFIGURATION );

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
// Set up the various Input Modules. These are stored in a single list
// rather than a lot of separate instances which would each be required to
// be made global.
$gModules = array();

require_once( PHP_DIR . "/environment/RequestParameterModule.php" );
$requestParameterModule = new RequestParameterModule( 'request-param' );
$gModules[ $requestParameterModule->getName() ] = $requestParameterModule;
// Must be done before sessions are set up
$requestParameterModule->parseQuery( $_SERVER[ 'QUERY_STRING' ] );

require_once( PHP_DIR . "/environment/FlowModule.php" );
$flowModule = new FlowModule( 'flow' );
$gModules[ $flowModule->getName() ] = $flowModule;

require_once( PHP_DIR . "/environment/ParameterModule.php" );
$paramsModule = new ParameterModule( 'params' );
$gModules[ $paramsModule->getName() ] = $paramsModule;

require_once( PHP_DIR . "/environment/GlobalModule.php" );
$globalsModule = new ParameterModule( 'global' );
$gModules[ $globalsModule->getName() ] = $globalsModule;

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
// The sitemap is the main repository of all that happens in Paloose.
// The master sitemap is the root of the stack and sub-sitemaps are pushed
// on the stack as required.
require_once( PHP_DIR . "/sitemap/Sitemap.php" );
require_once( PHP_DIR . "/sitemap/SitemapStack.php" );
$gSitemapStack = new SitemapStack();

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
// Create the session handler instance even though we may not need it.
// We have to set up the session module first.
ini_set( 'arg_separator.output','&amp;' );
require_once( PHP_DIR . "/environment/SessionModule.php" );
$sessionModule = new SessionModule( 'session' );
$gModules[ $sessionModule->getName() ] = $sessionModule;

require_once( PHP_DIR . "/environment/Sessions.php" );
$gSessionHandler = new Sessions();

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
// Each level of match has its own associated variables. These are stored as arrays
// on the variable stack.
require_once( PHP_DIR . "/sitemap/VariableStack.php" );
$gVariableStack = new VariableStack();

require_once( PHP_DIR . "/serialization/OutputStream.php" );
require_once( PHP_DIR . "/exception-handling/PalooseException.php" );
require_once( PHP_DIR . "/exception-handling/InternalException.php" );
require_once( PHP_DIR . "/exception-handling/UserException.php" );
require_once( PHP_DIR . "/exception-handling/RunException.php" );
require_once( PHP_DIR . "/exception-handling/ExitException.php" );
require_once( PHP_DIR . "/error-handling/ErrorPage.php" );

// Where all the work is done. We do not emerge from here until everything is finished.
$paloose = new Paloose();

// All done - so clean up and exit
LoggerManager::shutdown();
exit;

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
/**
 * The main Paloose class where everything happens.
 * 
 * This is run when the http request
 * is redirected here, courtesy of the .htaccess file in the user's site home folder.
 * It is called from the paloose.php file that is in the user's site folder, that
 * paloose.php file remains the only means to configure Paloose.
 *
 * @package paloose
 * @subpackage Paloose
 */
 
class Paloose {

   /** Logger instance for this class */   
   private $gLogger;
   
   /** Root sitemap */   
   private $gSitemap;

   //-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
   //-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
   /**
    * The constructor for the Paloose class.
    * 
    * This contains the entire functionality for
    * Paloose. It sets up all the required variables (global and local), initialises
    * the logger, parses and runs the root sitemap. All exception errors and early exit
    * track back to here.
    */
    
   function __construct()
   {
      global $gSitemapStack;
      global $gConfiguration;
      global $gModules;
      
      $rootSitemapDir = $_SERVER[ 'DOCUMENT_ROOT'];
      // Remove trailing path separator if required (all directory names are stored without
      // separator, which is put in as required when used).
      if ( substr( $rootSitemapDir, strlen( $rootSitemapDir ) - 1, 1 ) == Environment::getPathSeparator() ) {
         $rootSitemapDir = substr( $rootSitemapDir, 0, strlen( $rootSitemapDir ) - 1 );
      }

      $this->gLogger =& LoggerManager::getLogger( 'Paloose' );

      // Correct the URI and Query string to that requested. This is done
      // because of the redirect in the .htaccess file. Change that way of
      // doing things and you will have to change this routine.
      $matchableURI = Utilities::correctURI();

      $this->gLogger->debug( "Actual URI : " . $_SERVER[ 'REQUEST_URI' ] );
      $this->gLogger->debug( "Actual Query string : " . $_SERVER[ 'QUERY_STRING' ] );
      $this->gLogger->debug( "Matchable URI : " . $matchableURI );

      $requestParameterModule = $gModules[ 'request-param' ];
      // $requestParameterModule->parseQuery( $_SERVER[ 'QUERY_STRING' ] );
      $requestParameterModule->add( "resource", $matchableURI );

      // All errors and early exit exceptions come back to here
      try {
         // To start with the output stream is the standard one to reply to the client
         ob_start();
         $outputStream = new OutputStream( OutputStream::STANDARD_OUTPUT );

         // Set up the root sitemap, parse it and run it
         Environment::$gRootSitemapDir = $rootSitemapDir;
         $this->gSitemap = new Sitemap( $rootSitemapDir, $gConfiguration[ 'rootSiteMap' ], NULL, $outputStream );
         $gSitemapStack->push( $this->gSitemap );
         $this->gLogger->debug( "Parse sitemaps ----------------------------------" );
         $this->gSitemap->parse();
         $this->gLogger->debug( "Running sitemaps ----------------------------------" );
         // Not an internal request so mark false
         $this->gSitemap->run( $matchableURI, $_SERVER[ 'QUERY_STRING' ], false );
         $gSitemapStack->pop();
      } catch ( ExitException $e ) {
         // It is a little suspect to use exceptions to handle what is really
         // a normal exit (albeit an early exit), however it works quite nicely.
         $this->gLogger->debug( "Clean exit" );
         // Clean exit so finish
      } catch( InternalException $e ) {
         $this->gLogger->fatal( $e->getMessage() );
         $errorPage = new ErrorPage( $e->getMessage(), $e->getCode(), NULL );
         $errorPage->display();
      } catch( UserException $e ) {
         $this->gLogger->fatal( $e->getMessage() );
         $errorPage = new ErrorPage( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
         $errorPage->display();
      } catch( RunException $e ) {
         $this->gLogger->fatal( $e->getMessage() );
         $errorPage = new ErrorPage( $e->getMessage(), $e->getCode(), NULL );
         $errorPage->display();
      }
      // Flush the output buffer to the client before exiting (do not clean buffer afterwards!!)
      ob_end_flush();
   }

}
?>